@@ -1,12 +1,45 @@
+# (c) Deductive 2012-2020, all rights reserved
+# This code is licensed under MIT license (see license.txt for details)
 """
-(c) Dativa 2012-2020, all rights reserved
 
-This code is licensed under MIT license (see license.txt for details)
 
-Classes to cache query results locally or on S3
+These classes provide functionality to cache database results locally or on S3 and only re-execute the database query if the query SQL or parameters have changed.
 
-Saves with a unique hash to guarantee the same result and long as the queries are deterministic and
-the underlying data has not changed.
+Cache structure
+---------------
+
+Each query is cached to the path specified in cache_path with a generated filename that uniquely represents the querythat has been run, for example:
+
+``s3://cache_bucket/cache_prefix/get_results_5e5920011161b3081937572d25140237_ba98cb6452cf711ae0cec5c2dadc2585.csv.gz``
+
+This is made up as follows:
+
+* **s3://cache_bucket/cache_prefix/** - The path provided in the cache_path parameter when instantiating the class
+* **get_results** - The first part of the file name represents the name of the query run, e.g. get_tune_in_outcomes.sql
+* **5e5920011161b3081937572d25140237** - This is a unique hash calculated from the contents of the SQL file. When a new version of the file is created, a new hash will also be created.
+* **ba98cb6452cf711ae0cec5c2dadc2585** - This is a unique hash calculated from the parameters passed to the SQL file. A new file is created for each set of parameters passed to the file
+* **.csv.gz** - A fixed prefix, always set to “.csv.gz”
+
+Hash collisions
+---------------
+
+The hash is designed to be unique for each execution of the SQL file but there is about a 1 in a million chance that we’ll get the same hash for two different SQL files. If this occurs then the campaign results should look like total garbage so please double check the QA reports before sending to a customer.
+
+Archive queries
+---------------
+
+Each query that is run can be saved to an optionally specified sql_archive_path. In this location the full text of every executed query with the timestamp will be saved.
+
+Params
+------
+
+When creating a CachedQuery class you specify a set of parameters that are used across all queries executed using the class. This can be used to store common parameters for queries such as access tokens and secrets.
+
+Running multiple clients
+------------------------
+
+Before running the query, the client attempts to lock the file. This is either done by creating a .lock file with a similar name
+or by creating a row in DynamoDB to lock the file. The table in DynamoDB is "newtools.dynamo.doggo.lock" in us-east-1 and contains one entry per locked files. Entries are typically deleted after the lock is released.
 
 """
 
@@ -20,10 +53,11 @@ import shutil
 import hashlib
 
 from newtools.aws import S3Location
-from newtools.optional_imports import SqlClient, AthenaClient
-from newtools.doggo import DoggoLock, DoggoFileSystem
+from .sql_client import SqlClient
+from .athena import AthenaClient
+from newtools.doggo import DoggoLock, DoggoFileSystem, DynamoDogLock
 
-logger = logging.getLogger("dativa.attribution.cached_query")
+logger = logging.getLogger("newtools.cached_query")
 
 # Set S3fsMixin for backwards compatibility
 S3fsMixin = DoggoFileSystem
@@ -108,7 +142,8 @@ class BaseCachedQuery(BaseCachedClass):
     time_out_seconds = 1800
     maximum_age = 3600
 
-    def __init__(self, params=None, cache_path="", sql_archive_path=None, sql_paths=None, gzip=True):
+    def __init__(self, params=None, cache_path="", sql_archive_path=None, sql_paths=None, gzip=True,
+                 dynamodb_lock=True):
         """
         Cached query class
 
@@ -117,6 +152,7 @@ class BaseCachedQuery(BaseCachedClass):
         :param sql_archive_path: the path locally or on S3 to store archive SQL queries
         :param sql_paths: a list of paths to search for SQL queries
         :param gzip: if set, then results will be compression
+        :param dynamodb_lock: defaults to True, using Dynamo DB for locking in S3
         """
         super().__init__(cache_path)
 
@@ -128,6 +164,22 @@ class BaseCachedQuery(BaseCachedClass):
             self._sql_paths = self._sql_paths + sql_paths
 
         self._archive_path = sql_archive_path
+        self._aws_lock = DynamoDogLock if dynamodb_lock else DoggoLock
+
+    def _get_lock(self, path):
+        if path.startswith("s3://"):
+            return self._aws_lock(path,
+                                  wait_period=self.wait_period,
+                                  time_out_seconds=self.time_out_seconds,
+                                  maximum_age=self.maximum_age
+                                  )
+        else:
+            # short time outs for local file system
+            return DoggoLock(path,
+                             wait_period=self.wait_period,
+                             time_out_seconds=self.time_out_seconds,
+                             maximum_age=self.maximum_age
+                             )
 
     def _validate_args(self, args):
         """
@@ -239,10 +291,7 @@ class BaseCachedQuery(BaseCachedClass):
             data_hash=self._get_dict_hash(params, replacement_dict),
             suffix=".gz" if self._gzip else "")
 
-        with DoggoLock(path,
-                       wait_period=self.wait_period,
-                       time_out_seconds=self.time_out_seconds,
-                       maximum_age=self.maximum_age):
+        with self._get_lock(path):
             if self._exists(path):
                 self.dfs.rm(path)
 
@@ -293,12 +342,7 @@ class BaseCachedQuery(BaseCachedClass):
         self._log_parameters(params)
 
         if not self._exists(output_file):
-            # Get the file file lock
-            with DoggoLock(output_file,
-                           wait_period=self.wait_period,
-                           time_out_seconds=self.time_out_seconds,
-                           maximum_age=self.maximum_age):
-                # Check to see if it exists...
+            with self._get_lock(output_file):
                 if not self._exists(output_file):
                     # clean up any S3 files with the same prefix
                     for file in self.dfs.glob(output_file + "**"):
@@ -332,12 +376,38 @@ class BaseCachedQuery(BaseCachedClass):
 
 class CachedPep249Query(BaseCachedQuery):
     """
-    A CachedQuery class compatible with PEP249 classes
+    A CachedQuery class compatible with PEP249 classes.
+
+
+    The PEP249 query takes an additional argument containing any PEP249 compliant connection object.
+
+    Use the CachedPep249Query class with any PEP249 compliant database connector, like this:
+
+    .. code-block:: python
+
+        import pandas as pd
+        import sqlite3
+        from newtools import CachedPep249Query
+
+        pep_249_obj =  sqlite3.connect("mydb.sqlite")
+
+        q = CachedPep249Query(pep_249_obj,
+                              cache_path="s3://here-is-my-cache/",
+                              sql_paths=["local/path/with/sql/file", "another/local/path"],
+                              gzip=True)
+
+        results = q.get_results(sql_file="my_query.sql",
+                      output_prefix="test_query"
+                      )
+
+        df = pd.load_df(results, compression="gzip")
+
     """
     __sql = None
 
-    def __init__(self, pep_249_obj, params=None, cache_path="", sql_archive_path=None, sql_paths=None, gzip=True):
-        super().__init__(params, cache_path, sql_archive_path, sql_paths, gzip)
+    def __init__(self, pep_249_obj, params=None, cache_path="", sql_archive_path=None, sql_paths=None, gzip=True,
+                 dynamodb_lock=True):
+        super().__init__(params, cache_path, sql_archive_path, sql_paths, gzip, dynamodb_lock)
 
         self._sql = SqlClient(pep_249_obj,
                               logger=logger,
@@ -392,7 +462,34 @@ class CachedPep249Query(BaseCachedQuery):
 
 class CachedAthenaQuery(BaseCachedQuery):
     """
-    A CachedQuery class for AWS Athena
+    The cached AthenaQuery will execute queries against AWS Athena and cache results in csv.gz format.
+
+    Two special parameters must be passed to connect to Athena:
+
+    * **aws_region** - the AWS region
+    * **athena_db** - the Athena database to use
+
+    The Athena connection is created using Boto and uses the current AWS profile or AWS access keys and tokens.
+
+    Use the class as follows
+
+    .. code-block:: python
+
+        import pandas as pd
+        from newtools import CachedAthenaQuery
+
+        q = CachedAthenaQuery(cache_path="s3://here-is-my-cache/",
+                              sql_paths=["local/path/with/sql/file", "another/local/path"],
+                              params={"aws_region": "us-east-1",
+                                      "athena_db": "my_db"},
+                              gzip=True)
+
+
+        results = q.get_results(sql_file="my_query.sql",
+                      output_prefix="test_query"
+                      )
+
+        df = pd.load_df(results, compression="gzip")
 
     """
     __ac = None
