@@ -1,4 +1,4 @@
-# (c) 2012-2018 Dativa, all rights reserved
+# (c) 2012-2018 Deductive, all rights reserved
 # -----------------------------------------
 #  This code is licensed under MIT license (see license.txt for details)
 
@@ -15,7 +15,7 @@ from newtools.optional_imports import pandas as pd
 from newtools.optional_imports import numpy as np
 from newtools.optional_imports import boto3
 
-from newtools import PandasDoggo, FileDoggo, S3Location, DoggoFileSystem, DoggoLock, DoggoWait
+from newtools import PandasDoggo, FileDoggo, S3Location, DoggoFileSystem, DoggoLock, DoggoWait, DynamoDogLock
 from .base_test import BaseTest
 
 
@@ -215,7 +215,7 @@ class TestDoggos(unittest.TestCase):
         expected_df = pd.DataFrame({'e-mail': {0: 'n@gmail.com', 1: 'p@gmail.com', 2: 'h@gmail.com', 3: 's@gmail.com',
                                                4: 'l@gmail.com', 5: 'v@gmail.com', 6: None},
                                     'number': {0: 0, 1: 1, 2: 0, 3: 0, 4: 1, 5: 0, 6: 0}})
-        pth = S3Location('gd.dativa.test').join('generic', 'email_test.csv')
+        pth = S3Location('deductive-tools-testing').join('generic', 'email_test.csv')
         session = boto3.Session()
         fh = PandasDoggo(boto_session=session)
         df = fh.load(pth)
@@ -225,8 +225,8 @@ class TestDoggos(unittest.TestCase):
         df = pd.DataFrame({'e-mail': {0: 'n@gmail.com', 1: 'p@gmail.com', 2: 'h@gmail.com', 3: 's@gmail.com',
                                       4: 'l@gmail.com', 5: 'v@gmail.com', 6: None},
                            'number': {0: 0, 1: 1, 2: 0, 3: 0, 4: 1, 5: 0, 6: 0}})
-        pth = S3Location('gd.dativa.test').join('generic', 'email_test_copy.csv')
-        expected_pth = S3Location('gd.dativa.test').join('generic', 'email_test.csv')
+        pth = S3Location('deductive-tools-testing').join('generic', 'email_test_copy.csv')
+        expected_pth = S3Location('deductive-tools-testing').join('generic', 'email_test.csv')
         fh = PandasDoggo()
         fh.save(df, pth, index=None)
         with FileDoggo(pth) as p, FileDoggo(expected_pth) as p2:
@@ -240,7 +240,7 @@ class TestDoggos(unittest.TestCase):
         df = pd.DataFrame({'e-mail': {0: 'n@gmail.com', 1: 'p@gmail.com', 2: 'h@gmail.com', 3: 's@gmail.com',
                                       4: 'l@gmail.com', 5: 'v@gmail.com', 6: None},
                            'number': {0: 0, 1: 1, 2: 0, 3: 0, 4: 1, 5: 0, 6: 0}})
-        pth = S3Location('gd.dativa.test').join('generic', 'email_test_copy.csv.gz')
+        pth = S3Location('deductive-tools-testing').join('generic', 'email_test_copy.csv.gz')
 
         fh = PandasDoggo()
 
@@ -382,43 +382,64 @@ def check_lock(file):
         sleep(0.1)
 
 
-class TestDoggoLock(BaseTest):
+class BaseLockTest(BaseTest):
+    LockClass = None
+
     def test_lock_timeout(self):
-        file = os.path.join(self.tempdir, 'test_lock_timeout.tmp')
-        with DoggoLock(file, wait_period=1, time_out_seconds=5, maximum_age=10):
-            with self.assertRaises(TimeoutError):
-                with DoggoLock(file, wait_period=1, time_out_seconds=5, maximum_age=10):
-                    pass
+        if self.LockClass is not None:
+            file = os.path.join(self.tempdir, 'test_lock_timeout.tmp')
+            with self.LockClass(file, wait_period=1, time_out_seconds=5, maximum_age=20):
+                with self.assertRaises(TimeoutError):
+                    with self.LockClass(file, wait_period=1, time_out_seconds=2, maximum_age=20):
+                        pass
 
     def test_basic(self):
-        file = os.path.join(self.tempdir, 'test_basic.tmp')
-        check_lock(file)
-        check_lock(file)
+        if self.LockClass is not None:
+            file = os.path.join(self.tempdir, 'test_basic.tmp')
+            check_lock(file)
+            check_lock(file)
 
     def test_lock_ages(self):
-        file = os.path.join(self.tempdir, 'test_lock_ages.tmp')
-        lock_1 = DoggoLock(file, wait_period=1, time_out_seconds=2, maximum_age=5)
-        lock_2 = DoggoLock(file, wait_period=1, time_out_seconds=2, maximum_age=5)
+        if self.LockClass is not None:
+            file = os.path.join(self.tempdir, 'test_lock_ages.tmp')
+            lock_1 = self.LockClass(file, wait_period=1, time_out_seconds=2, maximum_age=5)
+            lock_2 = self.LockClass(file, wait_period=1, time_out_seconds=2, maximum_age=5)
 
-        lock_1.acquire()
+            lock_1.acquire()
+
+            with self.assertRaises(TimeoutError):
+                lock_2.acquire()
+
+            sleep(5)
 
-        with self.assertRaises(TimeoutError):
             lock_2.acquire()
 
-        sleep(5)
-
-        lock_2.acquire()
-
-        lock_1.release()
-        lock_2.release()
+            lock_1.release()
+            lock_2.release()
 
     def test_lock_multiprocess(self):
-        file = os.path.join(self.tempdir, 'test_lock_multiprocess.tmp')
-        pool = multiprocessing.Pool()
+        if self.LockClass is not None:
+            file = os.path.join(self.tempdir, 'test_lock_multiprocess.tmp')
+            pool = multiprocessing.Pool()
 
-        for _ in range(0, 100):
-            pool.apply_async(check_lock,
-                             (file,))
+            for _ in range(0, 100):
+                pool.apply_async(check_lock,
+                                 (file,))
 
-        pool.close()
-        pool.join()
+            pool.close()
+            pool.join()
+
+
+class TestDoggoLock(BaseLockTest):
+    LockClass = DoggoLock
+
+
+class TestDynamoDogLock(BaseLockTest):
+    LockClass = DynamoDogLock
+
+    def test_lock_create_dynamo_table(self):
+        boto3.resource("dynamodb", region_name="us-east-1").Table("newtools.dynamo.doggo.lock").delete()
+
+        file = os.path.join(self.tempdir, 'test_lock_ages.tmp')
+        with self.LockClass(file, wait_period=1, time_out_seconds=2, maximum_age=5):
+            pass
